#include <stdio.h>
#include <string.h>
#include "dtkcolorlegend.h"


static void dtk_color_legend_class_init (DtkColorLegendClass*klass);
static void dtk_color_legend_init (DtkColorLegend*color_legend);
static gint dtk_color_legend_expose (GtkWidget*widget, GdkEventExpose*event);
static gint dtk_color_legend_configure (GtkWidget*widget, GdkEventConfigure*event);
static void dtk_color_legend_finalize (GtkObject*object);
static void dtk_color_legend_draw (DtkColorLegend*legend);
static void dtk_color_legend_size_request (GtkWidget*widget, GtkRequisition*requisition);


GtkType dtk_color_legend_get_type ()
{
  static GtkType type = 0;

  if (!type)
  {
    static const GtkTypeInfo info = 
    {
      "DtkColorLegend",
      sizeof (DtkColorLegend),
      sizeof (DtkColorLegendClass),
      (GtkClassInitFunc) dtk_color_legend_class_init,
      (GtkObjectInitFunc) dtk_color_legend_init,
      NULL,
      NULL,
      (GtkClassInitFunc) NULL
    };
    type = gtk_type_unique (GTK_TYPE_DRAWING_AREA, &info);
  }
  return type;
}


static void dtk_color_legend_class_init (DtkColorLegendClass*klass)
{
  GtkObjectClass*object_class;
  GtkWidgetClass*widget_class;
  
  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;

  object_class->finalize = dtk_color_legend_finalize;
  
  widget_class->expose_event = dtk_color_legend_expose;
  widget_class->configure_event = dtk_color_legend_configure;
  widget_class->size_request = dtk_color_legend_size_request;
}


static void dtk_color_legend_size_request (GtkWidget*widget, GtkRequisition*requisition)
{
  g_return_if_fail (widget);
  g_return_if_fail (DTK_IS_COLOR_LEGEND (widget));
  
  requisition->width = 300;
  requisition->height = 300;
}

static void dtk_color_legend_init (DtkColorLegend*legend)
{
  legend->colors = 0;
  legend->num_colors = 0;
  legend->caption = 0;
  legend->values = 0;
  legend->pixmap = 0;
  legend->precision = 0;
  legend->margin = 4;
  legend->gc = 0;
}

GtkWidget*dtk_color_legend_new  (gint num_colors, gint*colors)
{
  DtkColorLegend*legend;
  int i;
  
  g_return_val_if_fail (num_colors > 0, 0);
  g_return_val_if_fail (colors, 0);

  legend = DTK_COLOR_LEGEND (gtk_type_new (dtk_color_legend_get_type ()));
  g_return_val_if_fail (legend, 0);
  
  legend->num_colors = num_colors;
  legend->colors = (GdkColor*) g_malloc (sizeof (GdkColor) * num_colors);
  g_return_val_if_fail (legend->colors, 0);

  legend->values = (gdouble*) g_malloc (sizeof (gdouble) * num_colors);
  g_return_val_if_fail (legend->values, 0);
  memset (legend->values, 0, sizeof (gdouble) * num_colors);

  for (i = 0; i < num_colors; i++)
  {
    legend->colors [i].red = (colors [i] >> 24) & 0xff;
    legend->colors [i].green = (colors [i] >> 16) & 0xff;
    legend->colors [i].blue = (colors [i] >> 8) & 0xff;
    
    legend->colors [i].red |= legend->colors [i].red << 8;
    legend->colors [i].green |= legend->colors [i].green << 8;
    legend->colors [i].blue |= legend->colors [i].blue << 8;
    
    if (!gdk_colormap_alloc_color (gtk_widget_get_colormap (GTK_WIDGET (legend)), legend->colors + i, FALSE, TRUE))
      return 0;
  }
  return GTK_WIDGET (legend);
}

static void dtk_color_legend_finalize (GtkObject*object)
{
  DtkColorLegend*legend;
  GtkObjectClass*parent_class;
  
  g_return_if_fail (object);
  g_return_if_fail (DTK_IS_COLOR_LEGEND (object));
  
  legend = DTK_COLOR_LEGEND (object);
  parent_class = GTK_OBJECT_CLASS (gtk_type_class (gtk_object_get_type ()));
  
  if (legend->num_colors > 0)
  {
    if (legend->colors)
    {
      gdk_colormap_free_colors (gtk_widget_get_colormap (GTK_WIDGET (legend)),
				legend->colors,
				legend->num_colors);
      g_free (legend->colors);
    }
    if (legend->values)
      g_free (legend->values);
  }
  
  if (legend->caption)
    g_free (legend->caption);

  if (legend->pixmap)
    gdk_pixmap_unref (legend->pixmap);

  if (legend->gc)
    gdk_gc_destroy (legend->gc);
  
  legend->colors = 0;
  legend->values = 0;
  legend->caption = 0;
  legend->pixmap = 0;
  legend->gc = 0;
  
  if (GTK_OBJECT_CLASS (parent_class)->finalize)
    (*GTK_OBJECT_CLASS (parent_class)->finalize) (object);
}

static gint dtk_color_legend_expose (GtkWidget*widget, GdkEventExpose*event)
{
  DtkColorLegend*legend;
  
  g_return_val_if_fail (widget, 0);
  g_return_val_if_fail (DTK_IS_COLOR_LEGEND (widget), 0);
  g_return_val_if_fail (event, 0);
  
  legend = DTK_COLOR_LEGEND (widget);
  
  if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget) && legend->pixmap)
  {
    gdk_draw_pixmap (widget->window,
		     widget->style->fg_gc [GTK_WIDGET_STATE (widget)],
		     legend->pixmap,
		     event->area.x, event->area.y,
		     event->area.x, event->area.y,
		     event->area.width, event->area.height);
  }
  return 0;
}

static gint dtk_color_legend_configure (GtkWidget*widget, GdkEventConfigure*event)
{
  DtkColorLegend*legend;
  
  g_return_val_if_fail (widget, 0);
  g_return_val_if_fail (DTK_IS_COLOR_LEGEND (widget), 0);
  g_return_val_if_fail (event, 0);
  
  legend = DTK_COLOR_LEGEND (widget);
  
  if (legend->pixmap)
    gdk_pixmap_unref (legend->pixmap);
  
  legend->pixmap = gdk_pixmap_new (widget->window, widget->allocation.width, widget->allocation.height, -1);
  g_return_val_if_fail (legend->pixmap, 0);

  if (!legend->gc)
    legend->gc = gdk_gc_new (widget->window);
  g_return_val_if_fail (legend->gc, 0);
  
  dtk_color_legend_draw (legend);
  return 1;
}

static void dtk_color_legend_draw (DtkColorLegend*legend)
{
  GtkWidget*w = GTK_WIDGET (legend);
  GdkRectangle r;
  gint caption_height, caption_width;
  gint value_width;
  gchar s [100];
  gchar f [10];
  gdouble biggest;
  gint i;

  if (!legend->pixmap)
    return;  

  g_return_if_fail (legend);
  g_return_if_fail (DTK_IS_COLOR_LEGEND (legend));
  g_return_if_fail (legend->num_colors > 0);
  
  gdk_draw_rectangle (legend->pixmap, w->style->bg_gc [GTK_WIDGET_STATE (w)], 1, 0, 0, w->allocation.width, w->allocation.height);

  if (!legend->caption)
    return;
  
  g_return_if_fail (legend->gc);
  
  caption_height = gdk_string_height (w->style->font, legend->caption) + legend->margin;
  caption_width = gdk_string_width (w->style->font, legend->caption);
  g_return_if_fail (caption_height > 0);
  g_return_if_fail (caption_width > 0);
  
  sprintf (f, "%s%d%s", "%.", legend->precision, "f");
  
  biggest = 0.0;
  for (i = 0; i < legend->num_colors; i++)
    if (biggest < legend->values [i])
      biggest = legend->values [i];
  
  sprintf (s, f, biggest);
  value_width = gdk_string_width (w->style->font, s);
  g_return_if_fail (value_width > 0);

  gdk_draw_string (legend->pixmap,
		   w->style->font, 
		   w->style->black_gc,
		   legend->margin,
		   caption_height,
		   legend->caption);

  r.width = w->allocation.width - value_width - 3 * legend->margin;
  r.height = (w->allocation.height - caption_height - 2 * legend->margin) / legend->num_colors;
  r.x = legend->margin;
  
  gdk_gc_set_fill (legend->gc, GDK_SOLID);

  for (i = 0; i < legend->num_colors; i++)
  {
    r.y = caption_height + legend->margin + r.height * i;
    
    gdk_gc_set_foreground (legend->gc, legend->colors + i);
    gdk_draw_rectangle (legend->pixmap, legend->gc, 1, r.x, r.y, r.width, r.height);
  }

  for (i = 0; i < legend->num_colors; i++)
  {
    sprintf (s, f, legend->values [i]);
    
    gdk_draw_string (legend->pixmap,
		     w->style->font, 
		     w->style->black_gc,
		     r.width + r.x + legend->margin,
		     caption_height + legend->margin + r.height * (i + 1),
		     s);
  }

  r.height *= legend->num_colors;
  r.y = caption_height + legend->margin;

  gdk_draw_rectangle (legend->pixmap, w->style->black_gc, 0, r.x, r.y, r.width, r.height);

  r.x = w->allocation.x;
  r.y = w->allocation.y;
  r.width = w->allocation.width;
  r.height = w->allocation.height;
  
  gtk_widget_draw (w, &r);
}

void dtk_color_legend_set (DtkColorLegend*legend, gchar*caption, gdouble*values, gint precision)
{
  g_return_if_fail (legend);
  g_return_if_fail (DTK_IS_COLOR_LEGEND (legend));
  g_return_if_fail (caption);
  g_return_if_fail (values);
  g_return_if_fail (precision >= 0);
  g_return_if_fail (legend->num_colors > 0);

  legend->caption = g_strdup (caption);
  legend->precision = precision;
  memcpy (legend->values, values, sizeof (gdouble) * legend->num_colors);
  
  dtk_color_legend_draw (legend);  
}
